---
layout: default
title:  "Roles and Abilities"
date:   2017-07-21 13:00:48 +0300
categories: [abilities, roles]
---

CASL forces you to think about what a user can actually do in the application and not who a user is in terms of roles, groups or whatever. This allows you to simply define user's abilities in JavaScript and define your own vision of how roles are translated into abilities. I will describe my vision.

In the simplest scenario, you will need to define abilities per role and share it between different users of the same role:

```js
const { AbilityBuilder } = require('casl')

const admin = AbilityBuilder.define(can => {
  can('manage', 'all')
})

const moderator = AbilityBuilder.define(can => {
  can('read', 'all')
  can('update', 'Post', { state: 'waitForModeration' })
})

const default = AbilityBuilder.define(can => {
  can('read', 'all')
})

export const ABILITIES = { admin, moderator, default }
```

And then use it in your application. For example, in expressjs:

```js
const ABILITIES = require('./abilities')

app.use(passport.initialize())
// ...
app.use((req, res, next) => {
  req.ability = req.user ? ABILITIES[req.user.roleName] : ABILITIES.default
})

app.patch('/posts/:id', (req, res, next) => {
  Post.findOne({ _id: req.params.id })
    .then(post => {
      req.ability.throwUnlessCan('update', post)
      post.set(req.body)
      return post.save()
    })
    .then(() => res.send({ ok: true }))
    .catch(next)
})
```

In more complex scenario, you will need to define abilities based on role and some user attributes, thus you will need to create a separate `Ability` instance per each request:

```js
async function defineAbilitiesFor(user) {
  const isValidSubscription = await user.isValidSubscription()

  return AbilityBuilder.define(can => {
    can('read', 'all')

    if (isValidSubscription) {
      can('manage', 'Domain', { owner: user.id })
    }
  })
}

app.use(async (req, res, next) => {
  req.ability = await defineAbilitiesFor(req.user)
  next()
})
```

In some complex cases it may be inefficient, so you will need to store abilities in database, session, cache or JWT token. See [Storing Abilities][storing-abilities] for details.

[storing-abilities]: {% post_url 2017-07-22-storing-abilities %}
